@@ -14,12 +14,14 @@ class CSVCallback(Callback):
         self,
         filename: str = None,
         filter_population: Callable[[list[Individual]], list[Individual]] = lambda x: x,
+        save_genotype_as_string: bool = True,
     ):
         if filename is None:
             filename = "evolution_results.csv"
         self.filename = filename
         self.filter_population = filter_population
         self.cumulative_time = 0.0
+        self.save_genotype_as_string = save_genotype_as_string
         self.write_header()
 
     def end_evolution(self):
@@ -28,17 +30,29 @@ class CSVCallback(Callback):
     def write_header(self):
         self.outfile = open(f"{self.filename}", "w", newline="")
         self.writer = csv.writer(self.outfile)
-        self.writer.writerow(
-            [
-                "genotype_as_str",
-                "fitness",
-                "depth",
-                "nodes",
-                "number_of_the_generation",
-                "time_since_the_start_of_the_evolution",
-                "seed",
-            ],
-        )
+        if self.save_genotype_as_string:
+            self.writer.writerow(
+                [
+                    "genotype_as_str",
+                    "fitness",
+                    "depth",
+                    "nodes",
+                    "number_of_the_generation",
+                    "time_since_the_start_of_the_evolution",
+                    "seed",
+                ],
+            )
+        else:
+            self.writer.writerow(
+                [
+                    "fitness",
+                    "depth",
+                    "nodes",
+                    "number_of_the_generation",
+                    "time_since_the_start_of_the_evolution",
+                    "seed",
+                ],
+            )
 
     def process_iteration(self, generation: int, population, time: float, gp):
         pop = self.filter_population(population)
@@ -52,13 +66,23 @@ class CSVCallback(Callback):
                 nodes = ind.genotype.gengy_nodes
             else:
                 nodes = -1
-            row = [
-                ind.genotype,
-                ind.fitness,
-                depth,
-                nodes,
-                generation,
-                self.cumulative_time,
-                gp.seed,
-            ]
+            if self.save_genotype_as_string:
+                row = [
+                    ind.genotype,
+                    ind.fitness,
+                    depth,
+                    nodes,
+                    generation,
+                    self.cumulative_time,
+                    gp.seed,
+                ]
+            else:
+                row = [
+                    ind.fitness,
+                    depth,
+                    nodes,
+                    generation,
+                    self.cumulative_time,
+                    gp.seed,
+                ]
             self.writer.writerow([str(x) for x in row])
