@@ -49,6 +49,7 @@ class GP:
         - timer_stop_criteria (bool): If set to True, the algorithm is stopped after the time limit (default = 60 seconds). Then the fittest individual is returned (default = False).
         - timer_limit (int): The time limit of the timer.
         - save_to_csv (str): Saves a CSV file with the details of all the individuals of all generations.
+        - save_genotype_as_string (bool): Turn this off if you don't want to safe all the genotypes as strings. This saves memory and a bit of time.
         - callbacks (List[Callback]): The callbacks to define what is done with the returned prints from the algorithm (default = []).
         -----
         Default as given in A Field Guide to GP, p.17, by Poli and Mcphee:
@@ -106,6 +107,7 @@ class GP:
         timer_stop_criteria: bool = False,  # TODO: This should later be generic
         timer_limit: int = 60,
         save_to_csv: str = None,
+        save_genotype_as_string: bool = True,
         callbacks: list[Callback] = None,
     ):
         assert population_size > (n_elites + n_novelties + 1)
@@ -167,7 +169,10 @@ class GP:
         self.force_individual = force_individual
 
         if save_to_csv:
-            c = CSVCallback(save_to_csv)
+            c = CSVCallback(
+                save_to_csv,
+                save_genotype_as_string=save_genotype_as_string,
+            )
             self.callbacks.append(c)
 
     def create_individual(self, depth: int):
