@@ -85,6 +85,8 @@ def is_abstract(t: type) -> bool:
 
 def is_terminal(t: type, l: set[type]) -> bool:
     """Returns whether a node is a terminal or not, based on the list of non terminals in the grammar"""
+    if isinstance(t, list):
+        return False
     if is_annotated(t):
         return all([is_terminal(inner, l) for inner in get_generic_parameters(t)])
     if not has_arguments(t):
